@@ -1,9 +1,10 @@
 """
-Provide the defaults and data for other modules.
-Specifically, the :class:`~powersystems.Generator` defaults
-are provided by :data:`generator_defaults`.
+Provide the defaults and configuration for other modules.
+`user_config` is treated as a global in minpower.
 """
+
 import logging
+from commonscripts import DotDict
 
 generator_kinds=[
     'generic',
@@ -27,17 +28,17 @@ generator_defaults=dict(
         ),
     is_controllable=dict(generic=True,wind=False),
     power=dict(generic=None),
-    
+
     Pmax=dict(
         generic=             500,
         coal=                550,
-        nuclear=            1117, 
+        nuclear=            1117,
         nggt=                 90,
         ngcc=                420,
         ngst   =             550,#assume equal to coal fired steam
         wind   =             100,
         ),
-    
+
     minuptime=dict(
         generic=0,
         coal=24,
@@ -47,7 +48,7 @@ generator_defaults=dict(
         ngst=24, #assume equal to coal fired steam
         wind=0,
         ),
-    
+
     mindowntime=dict(
         generic=0,
         coal=2,
@@ -57,7 +58,7 @@ generator_defaults=dict(
         ngst=12,#assume equal to coal fired steam
         wind=0,
         ),
-    
+
     fuelcost=dict(
         generic=    1,
         coal=       2.0,   #from EIA national average coal price: http://www.eia.gov/electricity/monthly/
@@ -67,10 +68,10 @@ generator_defaults=dict(
         nuclear =   0.65,  #from NWPP plan - Table I-34: Forecast nuclear fuel prices (2006$/MMBtu)
         wind=       0,
         ),
-    
-    
+
+
     startupcost=dict(
-        generic=              0, #if kind not specified, no startup cost 
+        generic=              0, #if kind not specified, no startup cost
 ##from Henry Louie thesis
 #        coal=            107800,
 #        nuclear=         283500,
@@ -89,12 +90,33 @@ generator_defaults=dict(
     )
 
 
-cost_load_shedding = 1000 #$/MWh
+available_solvers = ['gurobi','glpk']
 
-default_num_breakpoints=11
-default_hours_commitment=24
-default_hours_commitment_overlap=0
 
-optimization_solver='glpk'
-available_solvers = ['glpk'] #['glpk','gurobi','cplex']
-logging_level= logging.INFO
+user_config = DotDict(dict(
+    duals=False,
+    breakpoints=11,
+    hours_commitment=24,
+    hours_overlap=0,
+    cost_load_shedding=10000, #$/MWh
+    load_shedding_allowed=False,
+    dispatch_decommit_allowed=False,
+    solver='cplex',
+
+    reserve_fixed=0,
+    reserve_load_fraction=0.0,
+
+    faststart_resolve=False,
+
+    visualization=False,
+    logging_level=logging.DEBUG,
+    logging_filename=False,
+    problem_filename=False,
+    output_prefix=False,
+
+    scenarios=None,
+    deterministic_solve=False,
+    perfect_solve=False,
+    scenarios_directory=None,
+
+    ))
